#
# $Id: process_apache.pl,v 1.4 2001/12/17 12:10:47 fpmip Exp $
#

# Author : Tycho Fruru <tycho@fruru.com>

# Copyright (C) 2001 Tycho Fruru <tycho@fruru.com>
# Copyright (C) 2001 Conostix S.A. Luxembourg - Tycho Fruru <tycho@conostix.com>

# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#

print "Registering process_apache\n";

sub process_apache_access_log_1($$$$) {
	my $dbh = shift;
	my $entry = shift;
	my $table_name = shift;
	my $r_sql_hash = shift;
	
	# make local copies so that we don't pollute the global variables
	my %sql_hash = %{$r_sql_hash};
	
	my $rc = 1;
	
	my $apache_line = $entry->{'content'}->[0]->{'apache-line'};
	if (!defined($apache_line)) { 
		warn "There is no apache-line entry !";
		return 0;
	}
	
	if (ref($apache_line) eq "ARRAY") {
		my $index = 0;
		my $rc = 1;
		
		my $start_time = time();
		my $elapsed_time;
		my $speed;
		
		while (defined($apache_line->[$index]) && $rc==1) {
			
			if ($index % 1000 == 0) { 
				$elapsed_time = time()-$start_time;
				if ($index != 0) { 
					$speed = int($index / $elapsed_time);
				} else {
					$speed = 0;
				}
 				show_msg(3,"$index entries processed in $elapsed_time seconds, $speed per second.\n");
			}
		
		 	my %apache_sql_hash = %sql_hash;
			
			my $apache_virtual = "";
			my $apache_host;
			my $apache_logname;
			my $apache_username;
			my $apache_time;
			my $apache_url;
			my $apache_status;
			my $apache_bytes;
			my $apache_referer = "-";
			my $apache_user_agent = "-";
			
			my $decoded_apache_line = decode_entry($apache_line->[$index]);
			
			if ($decoded_apache_line =~ /^(\S+) (\S+) (\S+) \[([^\]]+)\] \"(.*)\" (\d+) (\S+) \"(.*)\" \"(.*)\"$/) {
# Combined log : "%h %l %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-agent}i\""
				$apache_host = $1;
				$apache_logname = $2;
				$apache_username = $3;
				$apache_time = $4;
				$apache_url = $5;
				$apache_status = $6;
				$apache_bytes = $7;
				$apache_referer = $8;
				$apache_user_agent = $9;
			} elsif ($decoded_apache_line =~ /^(\S+) (\S+) (\S+) \[([^\]]+)\] (\S+) \"(.*)\" (\d+) (\S+)$/) {
# CLF with virtual host : %v %h %l %u %t \"%r\" %>s %b
				$apache_virtual = $1;
				$apache_host = $2;
				$apache_logname = $3;
				$apache_username = $4;
				$apache_time = $5;
				$apache_url = $6;
				$apache_status = $7;
				$apache_bytes = $8;
			} elsif ($decoded_apache_line =~ /^(\S+) (\S+) (\S+) \[([^\]]+)\] \"(.*)\" (\d+) (\S+)$/) {
# CLF : %h %l %u %t \"%r\" %>s %b
				$apache_host = $1;
				$apache_logname = $2;
				$apache_username = $3;
				$apache_time = $4;
				$apache_url = $5;
				$apache_status = $6;
				$apache_bytes = $7;
			} else {
				warn "No applicable apache_access_log format for ".$decoded_apache_line;
				return 0;
			};
			
			if ($apache_bytes eq "-") {
				$apache_bytes = 0;
			}
			
			$apache_sql_hash{"message"}=$decoded_apache_line;
			$apache_sql_hash{"virtual_host"}=$apache_virtual;
			$apache_sql_hash{"src_host"}=$apache_host;
			$apache_sql_hash{"remote_logname"}=$apache_logname;
			$apache_sql_hash{"remote_username"}=$apache_username;

# apache time format : 17/May/2001:22:56:54 +0200 , directly understood by postgresql

			if ($apache_time =~ /^\d+\/\w+\/\d+:\d+:\d+:\d+ [+-]\d+$/) {
				$apache_sql_hash{"time"}=$apache_time;
			} else {
				show_msg(4,"Apache time spec $apache_time not acceptable in entry nr. $index\n");
			}
			
			$apache_sql_hash{"url"}=$apache_url;			
			$apache_sql_hash{"status"}=$apache_status;
			$apache_sql_hash{"bytes_sent"}=$apache_bytes;
			$apache_sql_hash{"referer"}=$apache_referer;
			$apache_sql_hash{"user_agent"}=$apache_user_agent;
			
			$rc = insert_data_into_table($dbh,$table_name,\%apache_sql_hash);
			$index++;
		}
	} else { 
		warn "There are no apache-lines in this file !"; 
		$rc = 0; 
	}
			
	show_msg(5,"End of apache_access_log_1\n");
	
	return $rc; # 1 is ok, 0 is not ok
}

return 1;
