#
# $Id: process_syslog.pl,v 1.1 2001/12/03 10:06:07 fpmip Exp $
#

# Author : Tycho Fruru <tycho@fruru.com>

# Copyright (C) 2001 Tycho Fruru <tycho@fruru.com>
# Copyright (C) 2001 Conostix S.A. Luxembourg - Tycho Fruru <tycho@conostix.com>

# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#

use Time::Local;

print "Registering process_syslog\n";

sub process_syslog_events_1($$$$) {
	my $dbh = shift;
	my $entry = shift;
	my $table_name = shift;
	my $r_sql_hash = shift;
	
	# make local copies so that we don't pollute the global variables
	my %sql_hash = %{$r_sql_hash};
	
	my $syslog_line = $entry->{'content'}->[0]->{'syslog-line'};

	if (!defined($syslog_line)) { 
		warn "There is no syslog-line entry !";
		return 0;
	}
	
	my $rc = 1;
	
	my %syslog_line_attributes;
	
	if (ref($syslog_line) eq "ARRAY") {
		my $index = 0;
		
		my $start_time = time();
		my $elapsed_time;
		my $speed;
		
		while (defined($syslog_line->[$index]) && $rc==1) {

			if ($index % 1000 == 0) { 
				$elapsed_time = time()-$start_time;
				if ($index != 0) { 
					$speed = int($index / $elapsed_time);
				} else {
					$speed = 0;
				}
 				show_msg(3,"$index entries processed in $elapsed_time seconds, $speed per second.\n");
			}

		 	my %syslog_sql_hash = %sql_hash;
			my $syslog_decoded_line=decode_entry($syslog_line->[$index]);
			
			%syslog_line_attributes =
		  		split_one_syslog_line_1($syslog_decoded_line);

			my $current_syslog_line_attribute;
			
			foreach $current_syslog_line_attribute (keys %syslog_line_attributes) {	
				$syslog_sql_hash{$current_syslog_line_attribute} = 
						$syslog_line_attributes{$current_syslog_line_attribute};
			}

			$syslog_sql_hash{'message'}         = $syslog_decoded_line;
			
			$rc = insert_data_into_table($dbh,$table_name,\%syslog_sql_hash);
			$index++;
		}
	} else { 
		warn "There are no syslog-lines in this file !"; 
		$rc = 0; 
	}
			
	show_msg(5,"End of syslog_event_1\n");
	return $rc;
}

sub split_one_syslog_line_1($) {
	my $syslog_line = shift;
	my %syslog_line;
	
# example syslog line:
# Oct  8 13:55:15 kungfoo /sbin/hotplug: invoke /etc/hotplug/usb.agent
#	print "Syslog line : $syslog_line\n";
	if ($syslog_line =~ /^\s*(\w+\s+\d+\s+\d+:\d+:\d+)\s+(\w+)\s+(([^:]+)\:)?\s*(\w*.*)$/) {
	
		$syslog_line{'syslog_date'} = $1;
		$syslog_line{'host'} = $2;
		$syslog_line{'logging_program'} = $4;
		$syslog_line{'pid'} = 0;
		$syslog_line{'log_message'} = $5;
	}
	
	if (!defined($syslog_line{'logging_program'})) {
		$syslog_line{'logging_program'} = "unknown";
	}
	
	if ($syslog_line{'logging_program'} =~ /^(.+)\[(\d+)\]$/) {
		$syslog_line{'logging_program'} = $1;
		$syslog_line{'pid'} = $2;
	}
	
	if ($syslog_line{'syslog_date'} =~ /^(\w+)\s+(\d+)\s+(\d+:\d+:\d+)$/) {
		my $syslog_month = lc($1);
		my $syslog_day = $2;
		my $syslog_time = $3;
		
		# heuristic to get the right year (which is not included in syslog lines !)
		# try the current year, one year ahead and one year behind
		# the date which is the nearest from the current one is supposed to be the right one
		
		my ($sec,$min,$hour,$mday,$current_mon,$current_year,$wday,$yday) =
                                                           gmtime(time);
							   
		$current_year+=1900; # use the correct 4 digit version
		
		my %months=('jan'=>0,'feb'=>1,'mar'=>2,'apr'=>3,'may'=>4,'jun'=>5,
			    'jul'=>6,'aug'=>7,'sep'=>8,'oct'=>9,'nov'=>10,'dec'=>11);
			    
		$syslog_month = $months{$syslog_month};
		if (!$syslog_month) { $syslog_month = 0 };
		
		my $current_seconds = time;
		my $syslog_current_year = timegm(0,0,0,$syslog_day,$syslog_month,$current_year);
		my $syslog_next_year = timegm(0,0,0,$syslog_day,$syslog_month,$current_year+1);
		my $syslog_previous_year = timegm(0,0,0,$syslog_day,$syslog_month,$current_year-1);
		
		my $difference_current = abs($current_seconds - $syslog_current_year);
		
		my $difference_next = abs($current_seconds - $syslog_next_year);
		my $difference_previous = abs($current_seconds - $syslog_previous_year);
		
		my $guessed_year;
		
		if (($difference_current < $difference_next) &&
		    ($difference_current < $difference_previous)) {
			$guessed_year = $current_year;
		} elsif ($difference_previous < $difference_next) {
			$guessed_year = $current_year-1;
		} else {
			$guessed_year = $current_year+1;
		}
		
		$syslog_month+=1; # to get back to human months
		$syslog_line{'time'}="$guessed_year-$syslog_month-$syslog_day $syslog_time";
			
	}
	
	return %syslog_line;
}

return 1;
