#
# $Id: update-db-backend.pl,v 1.3 2001/12/15 21:24:12 fpmip Exp $
#

# Author : Tycho Fruru <tycho@fruru.com>

# Copyright (C) 2001 Tycho Fruru <tycho@fruru.com>
# Copyright (C) 2001 Conostix S.A. Luxembourg - Tycho Fruru <tycho@conostix.com>

# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#

use XML::Simple;
use Data::Dumper;	# for debugging those XML data structures

sub update_db_backend() {
	show_msg(5,"Beginning of update_db_backend\n");
	show_msg(4,"Getting all agents which this db-backend manages\n");
	
	my ($dbh_agents,$sth_agents,$agent_id,$agent_type_name);
	
	$dbh_agents = DBI->connect($DB_CONNECT_STRING,$DB_USERNAME,$DB_PASSWORD) or
				die "Database connection failed";
	$dbh_agents->{AutoCommit} = 0;
	
	$sth_agents = $dbh_agents->prepare(
			"select agent_id,agent_type_name from agents") or
				die "select agent_id failed";
	$sth_agents->execute or die "select agent_id execution failed";

	show_msg(5,"Opening database connection for the updaters ...\n");

	my $dbh;
	$dbh = DBI->connect($DB_CONNECT_STRING,"","") or
				die "Database connection failed";
	$dbh->{AutoCommit} = 0;		

	while (($agent_id,$agent_type_name) = $sth_agents->fetchrow_array) {
		show_msg(4,"Starting on agent $agent_id, ".
			   "type $agent_type_name\n");

# first we process status messages, which are generic for all agents
# then we process agent-specific data (eg. webserver logs or snort alerts)

		process_messages($agent_id,$agent_type_name,$dbh);

	}
	
	$sth_agents->finish;
	$dbh_agents->disconnect;
	$dbh->disconnect;
}


sub process_messages($$$) {
	my $agent_id = shift;
	my $agent_type_name = shift;
	my $dbh = shift;
	show_msg(4,"Beginning of process_messages (agent $agent_id)\n");
	my (@ok_files,$current_file);
	my $current_directory;
	foreach $current_directory ('status','events') {
		show_msg(6,"Getting all files in the $current_directory dir\n");
		@ok_files = get_all_ok_files("$XML_DOWNLOAD_DIRECTORY/$agent_id/$current_directory");
		while ($current_file = shift(@ok_files)) {
			show_msg(5,"Treating file $agent_id/$current_directory/$current_file\n");
			my $rc = process_one_file($agent_id,$agent_type_name,$dbh,
						  "$XML_DOWNLOAD_DIRECTORY/$agent_id/$current_directory/$current_file");
			if ($rc) { 
				show_msg(5,"OK, moving the file to the archive location\n");
				if (-e "$XML_ARCHIVE_DIRECTORY/$agent_id/$current_directory/$current_file" ) {
					show_msg(1,"Archive file $XML_ARCHIVE_DIRECTORY/$agent_id/$current_directory/$current_file already exists...");
				}
				rename "$XML_DOWNLOAD_DIRECTORY/$agent_id/$current_directory/$current_file",
			       		"$XML_ARCHIVE_DIRECTORY/$agent_id/$current_directory/$current_file" or
						die "Cannot move treated file to $XML_ARCHIVE_DIRECTORY/$agent_id/$current_directory";
				unlink "$XML_DOWNLOAD_DIRECTORY/$agent_id/$current_directory/${current_file}.ok";
			} else {
				show_msg(1,"Problem with file $XML_DOWNLOAD_DIRECTORY/$agent_id/$current_directory/$current_file\n");
				if (-e "$XML_PROBLEM_DIRECTORY/$agent_id/$current_directory/$current_file" ) {
					show_msg(1,"Problem file $XML_ARCHIVE_DIRECTORY/$agent_id/$current_directory/$current_file already exists...");
				}
				rename "$XML_DOWNLOAD_DIRECTORY/$agent_id/$current_directory/$current_file",
					"$XML_PROBLEM_DIRECTORY/$agent_id/$current_directory/$current_file" or 
						die "Cannot move problematic file to $XML_PROBLEM_DIRECTORY/$agent_id/$current_directory";
				unlink "$XML_DOWNLOAD_DIRECTORY/$agent_id/$current_directory/${current_file}.ok";
			}
			show_msg(5,"Finished file $agent_id/$current_directory/$current_file\n");
		}
	}	
}

sub get_all_ok_files($) {
# returns a list with all files which have a corresponding OK file
	show_msg(7,"Beginning get_all_ok_files ");
	my $directory = shift;
	show_msg(7,"in directory $directory\n");
	opendir DIRHANDLE,$directory or
		die "Cannot open directory $directory";
	
	my $current_file;
	my @ok_files=();
	
	while ($current_file = readdir DIRHANDLE) {
		show_msg(10,"Current file : $current_file\n");
		if (($current_file eq ".") ||
		    ($current_file eq "..")) {
		    	next;
		}
		show_msg(10,"Treating file $current_file\n");
		if ($current_file =~ /^([0-9a-zA-Z_\-\.]*)\.ok$/) {
			if (-f "$directory/$1") {
				# The corresponding status file exists
				show_msg(7,"File $current_file is an OK file for file $1\n");
				push @ok_files,$1;
			} else { 
				show_msg(2,"File $current_file is an OK file without real file $1\n");
			}
		}	
	}
	closedir DIRHANDLE;
	return @ok_files;
} 

sub process_one_file($$$$) {
	my $agent_id = shift;
	my $agent_type_name = shift;
	my $dbh = shift;
	my $filename = shift;
	if (not -f "$filename") {
		show_msg(1,"process_one_file: The file $filename does not exist!\n");
		return 0;
	}
	my $file;
	
	eval { 
		$file = XMLin($filename,forcearray=>1,keyattr=>[],contentkey=>'encoded-content');
	};
	
	if ($@) { 
		show_msg(1,"Problem parsing the XML content for file $filename");
		return 0;
	}
	
	my $module_ip = check_ip($file->{agent}->[0]->{ip}->[0]);
	if (!defined($module_ip)) {
		show_msg(1,"Strange module_ip in $filename\n");
		return 0;
	};
	my $agent_time = check_string($file->{agent}->[0]->{date}->[0]);
	if (!defined($agent_time)) {
		show_msg(1,"Strange module_date in $filename\n");
		return 0;
	};
	my $module_name = check_string_nospace($file->{agent}->[0]->{name}->[0]);
	if (!defined($module_name)) { 
		show_msg(1,"Strange module_name in $filename\n");
		return 0;
	};
	my $wrapper_id = $file->{wrapperid}->[0];
	if (!defined($wrapper_id)) {
		$wrapper_id = 0;
	}
	$wrapper_id = check_numeric($wrapper_id);
	if (!defined($wrapper_id)) {
		show_msg(1,"Strange wrapper_id in $filename\n");
		return 0;
	}
	my $generation_id = check_string($file->{generationid}->[0]);
	if (!defined($generation_id)) {
		show_msg(1,"Strange generation_id in $filename\n");
		return 0;
	};
	my $sequence_id = check_numeric($file->{sequenceid}->[0]);
	if (!defined($sequence_id)) {
		show_msg(1,"Strange sequence_id in $filename\n");
		return 0;
	};
	my $transac_id = check_numeric($file->{transacid}->[0]);
	if (!defined($transac_id)) {
		show_msg(1,"Strange transac_id in $filename\n");
		return 0;
	};
		
	my $sth=$dbh->prepare("insert into logfiles 
		   (agent_id,wrapper_id,generation_id, sequence_id,transac_id,agent_time) values
		   (?,?,?,?,?,?)"
		) || die "Cannot prepare insert logfile data in table logfiles";

	my $logfile_insert_problem=0;
	
	$sth->execute($agent_id,$wrapper_id,$generation_id,$sequence_id,$transac_id,$agent_time) or $logfile_insert_problem=1;
	
	$sth->finish;

	if ($logfile_insert_problem) {
		warn "$agent_id, $wrapper_id, $generation_id, $sequence_id cannot be inserted into table logfiles ... it probably exists already";
		return 0;
	}

	
	$sth=$dbh->prepare("select logfile_id from logfiles where agent_id=? ".
				"and wrapper_id=? and generation_id=? ".
				"and sequence_id=?");
	$sth->execute($agent_id,$wrapper_id,$generation_id,$sequence_id) or die "select logfile_id execution failed";
	
	my $logfile_id = $sth->fetchrow_array or die "logfile_id for $agent_id, $wrapper_id, $generation_id, $sequence_id not found - not normal !";
	
	$sth->finish;
	
	show_msg(7,"Logfile id : $logfile_id\n");
	show_msg(7,"Sensor_type_name : $agent_type_name\n");
	
# Now we must process all data entries

	my $rc = 1;
	
	if (ref($file->{data}->[0]->{entry}) eq "ARRAY") {
		# there are multiple entries
		show_msg(8,"The reference is to an ARRAY containing multiple entries\n");
		
		my $index = 0;
		while (defined($file->{data}->[0]->{entry}->[$index] && $rc==1)) {
			$rc = process_one_entry($agent_id,$agent_type_name,$dbh,$logfile_id,$file->{data}->[0]->{entry}->[$index]);
			$index++;
		};
		
		
	} else {
		show_msg(1,"There are no entries in this file, committing just the existence of the file");
	};

	if ($rc == 1) {
		if ($DO_COMMIT == 1) {
			show_msg(5,"Performing commit...\n");
			$dbh->commit;
		}
	} else {
		show_msg(5,"Performing rollback...\n");
		$dbh->rollback;
	}
	
	return $rc;

};


sub process_one_entry($$$$$) {
# agent_id
# agent_type_name
# dbh
# logfile_id
# ->{data}->{entry}->[index]
	
	my $agent_id = shift;
	my $agent_type_name = shift;
	my $dbh = shift;
	my $logfile_id = shift;
	my $entry = shift;

	show_msg(7,"Beginning of process_one_entry\n");
		
#	print Dumper($entry);
	show_msg(7,"End of process_one_entry\n");
	
	my $entry_type = check_string($entry->{type}->[0]);
	my $entry_category = check_string($entry->{category}->[0]);
	my $entry_version = check_numeric($entry->{version}->[0]);
	
	show_msg(6,"type $entry_type, category $entry_category, version $entry_version\n");
	show_msg(6,"trying to find the right tablename : ");
	
	my $sth=$dbh->prepare("select table_name from table_names where ".
			      "agent_type_name = ? and ".
			      "entry_type       = ? and ".
			      "entry_category   = ? and ".
			      "entry_version    = ?");
	$sth->execute($agent_type_name,$entry_type,$entry_category,$entry_version);
	
	my $table_name;
	
	if (not ($table_name = $sth->fetchrow_array)) { 
		warn "Cannot obtain the right table name!";
		return 0;
	}
	
	show_msg(6,"$table_name\n");
	
	my $processing_function_name = "process_$table_name";

# prepare the SQL fields and values we know already

	my $sql_hash;
	
	$sql_hash->{"agent_id"}       = $agent_id;
	$sql_hash->{"event_type"}     = $entry_type;
	$sql_hash->{"event_category"} = $entry_category;
	$sql_hash->{"event_version"}  = $entry_version;
	$sql_hash->{"logfile_id"}     = $logfile_id;
	
# process all entries using the right subroutine
	my $result = &$processing_function_name($dbh,$entry,$table_name,$sql_hash);
	
	if ($result == 0) {
		warn "processing of $processing_function_name did not succeed !";
	}
	
	return $result;
}

sub insert_data_into_table($$$) {
	my $dbh = shift;
	my $table_name = shift;
	my $r_hash = shift;
	
	my @sql_fields = keys %{$r_hash};
	my @sql_values = values %{$r_hash};
	
	my $sql_statement = "insert into $table_name (".
				join(",",@sql_fields).
			    ") values (".
				join(",", split(//,"?"x(scalar(@sql_values)))).
			    ")";
	
	my $return_code = 0;
	
	my $sth=$dbh->prepare($sql_statement);
	if (!$sth) {
		show_msg(1,"Problem preparing $sql_statement with values ".join(',',@sql_values));
		eval { $sth->finish; };
		return 0;
	} 
	
	my $rc = $sth->execute(@sql_values);
	if (!$rc) {
		show_msg(1,"Problem executing $sql_statement with values ".join(',',@sql_values));
		eval { $sth->finish; };
		return 0;	
	}
	$sth->finish or return 0;
	return $rc;
}

return 1;
